from rave_python.rave_exceptions import RaveError, IncompletePaymentDetailsError, MobileChargeError, TransactionVerificationError, TransactionValidationError, ServerError
from rave_python.rave_payment import Payment
from rave_python.rave_misc import generateTransactionReference
import json
import webbrowser

class Francophone(Payment):
    
    def __init__(self, publicKey, secretKey, production, usingEnv):
        super(Francophone, self).__init__(publicKey, secretKey, production, usingEnv)


    # returns true if further action is required, false if it isn't    
    # def _handleChargeResponse(self, response, txRef, request=None):
    #     """ This handles charge responses """
    #     res =  self._preliminaryResponseChecks(response, MobileChargeError, txRef=txRef)

    #     responseJson = res["json"]
    #     flwRef = res["flwRef"]

    #     # Checking if there is redirect url

    #     if responseJson["data"]["data"].get("redirect_url", "N/A") == "N/A":
    #         redirectUrl = None
    #     else:
    #         redirectUrl = responseJson["data"]["data"]["redirect_url"]

    #     # If all preliminary checks passed
    #     if not (responseJson["data"].get("chargeResponseCode", None) == "00"):
    #         # Otherwise we return that further action is required, along with the response
    #         # suggestedAuth = responseJson["data"].get("suggested_auth", None)
    #         return {
    #                     "error": False, 
    #                     "status": responseJson["status"],  
    #                     "message": responseJson["message"],
    #                     "code": responseJson["data"]["code"],
    #                     "transaction status": responseJson["data"]["status"],
    #                     "ts": responseJson["data"]["ts"],
    #                     "link": responseJson["data"]["link"]
    #                 }
    #     else:
    #         return {"error": False, "status": responseJson["status"],  "validationRequired": False, "txRef": txRef, "flwRef": flwRef, "suggestedAuth": None, "redirectUrl": redirectUrl}
    
    # Charge mobile money function
    def charge(self, accountDetails, hasFailed=False):
        """ This is the charge call for central francophone countries.
             Parameters include:\n
            accountDetails (dict) -- These are the parameters passed to the function for processing\n
            hasFailed (boolean) -- This is a flag to determine if the attempt had previously failed due to a timeout\n
        """

        feature_name = "Initiate-Francophone-mobile-money-charge"
        endpoint = self._baseUrl + self._endpointMap["account"]["charge"]
        # It is faster to add boilerplate than to check if each one is present
        accountDetails.update({"payment_type": "mobilemoneyfrancophone", "is_mobile_money_franco":"1"})
        
        # If transaction reference is not set 
        if not ("txRef" in accountDetails):
            accountDetails.update({"txRef": generateTransactionReference()})
        # If order reference is not set
        if not ("orderRef" in accountDetails):
            accountDetails.update({"orderRef": generateTransactionReference()})
        # Checking for required account components
        # requiredParameters = ["amount", "email", "phonenumber", "IP", "redirect_url"]
        requiredParameters = ["amount"]
        return super(Francophone, self).charge(feature_name, accountDetails, requiredParameters, endpoint)



from rave_python.rave_exceptions import AccountChargeError
from rave_python.rave_misc import generateTransactionReference
from rave_python.rave_payment import Payment


class Account(Payment):
    """ This is the rave object for account transactions. It contains the following public functions:\n
        .charge -- This is for making an account charge\n
        .validate -- This is called if further action is required i.e. OTP validation\n
        .verify -- This checks the status of your transaction\n
    """
    def _handleChargeResponse(self, response, txRef, request=None):
        """ This handles account charge responses """
        # This checks if we can parse the json successfully
        res =  self._preliminaryResponseChecks(response, AccountChargeError, txRef=txRef)

        response_json = res['json']
        # change - added data before flwRef
        response_data = response_json['data']
        flw_ref = response_data['flwRef']

        # If all preliminary checks are passed
        data = {
            'error': False,
            'validationRequired': True,
            'txRef': txRef,
            'flwRef': flw_ref,
            'authUrl': None,
        }
        if response_data.get("chargeResponseCode") != "00":
            # If contains authurl
            data['authUrl'] = response_data.get("authurl")  # None by default
        else:
            data['validateInstructions'] = response_data['validateInstructions']
        return data

    # Charge account function
    def charge(self, accountDetails, hasFailed=False):
        """ This is the ghMobile charge call.\n
             Parameters include:\n
            accountDetails (dict) -- These are the parameters passed to the function for processing\n
            hasFailed (boolean) -- This is a flag to determine if the attempt had previously failed due to a timeout\n
        """

        # setting the endpoint
        endpoint = self._baseUrl + self._endpointMap['account']['charge']
        feature_name = "Initiate-Account-charge"

        # It is faster to just update rather than check if it is already present
        accountDetails.update({'payment_type': 'account'})

        # Generate transaction reference if txRef doesn't exist
        accountDetails.setdefault('txRef', generateTransactionReference())

        # Checking for required account components
        requiredParameters = ['accountbank', 'accountnumber', 'amount', 'email', 'phonenumber', 'IP']

        return super().charge(feature_name, accountDetails, requiredParameters, endpoint)

    def validate(self, flwRef, otp):
        endpoint = self._baseUrl + self._endpointMap['account']['validate']
        feature_name = "Account-charge-verify"
        return super().validate(feature_name, flwRef, endpoint)

    def verify(self, txRef):
        endpoint = self._baseUrl + self._endpointMap['account']['verify']
        feature_name = "Account-charge-verify"
        return super().verify(feature_name, txRef, endpoint)


import json, requests, copy
from rave_python.rave_base import RaveBase
from rave_python.rave_misc import checkIfParametersAreComplete
from rave_python.rave_exceptions import ServerError, IncompleteAccountDetailsError, AccountCreationError, AccountStatusError

class VirtualAccount(RaveBase):
    def __init__(self, publicKey, secretKey, production, usingEnv):
        self.headers = {
            'content-type' : 'application/json'
        }
        super(VirtualAccount, self).__init__(publicKey, secretKey, production, usingEnv)

    def _preliminaryResponseChecks(self, response, TypeOfErrorToRaise, name):
        #check if we can get json
        try:
            responseJson = response.json()
        except:
            raise ServerError({"error": True, "name": name, "errMsg": response})

        #check for data parameter in response 
        if not responseJson.get("data", None):
            raise TypeOfErrorToRaise({"error": True, "name": name, "errMsg": responseJson.get("message", "Server is down")})

        #check for 200 response
        if not response.ok:
            errMsg = response["data"].get("message", None)
            raise TypeOfErrorToRaise({"error": True, "errMsg": errMsg})

        return responseJson

    def _handleCreateResponse(self, response, accountDetails):
        responseJson = self._preliminaryResponseChecks(response, AccountCreationError, accountDetails["email"])

        if responseJson["status"] == "success":
            return {"error": False, "id": responseJson["data"].get("id", None), "data": responseJson["data"] }

        else:
            raise AccountCreationError({"error": True, "data": responseJson["data"]})

    # def _handleCardStatusRequests(self, type, endpoint, isPostRequest=False, data=None):
    #     #check if response is a post response
    #     if isPostRequest:
    #         response = requests.post(endpoint, headers=self.headers, data=json.dumps(data))
    #     else:
    #         response = requests.get(endpoint, headers=self.headers)
        
    #     #check if it can be parsed to JSON
    #     try:
    #         responseJson = response.json()
    #     except:
    #         raise ServerError({"error": True, "errMsg": response.text})

    #     if response.ok:
    #         return {"error": False, "returnedData": responseJson}
    #     else:
    #         raise AccountStatusError(type, {"error": True, "returnedData": responseJson })


    #function to create a virtual card 
    #Params: cardDetails - a dict containing email, is_permanent, frequency, duration, narration
    def create(self, accountDetails):

        # feature logic
        accountDetails = copy.copy(accountDetails)
        accountDetails.update({"seckey": self._getSecretKey()})
        requiredParameters = ["email", "narration"]
        checkIfParametersAreComplete(requiredParameters, accountDetails)
        endpoint = self._baseUrl + self._endpointMap["virtual_account"]["create"]
        response = requests.post(endpoint, headers=self.headers, data=json.dumps(accountDetails))

        #feature logging
        if response.ok == False:
            tracking_endpoint = self._trackingMap
            responseTime = response.elapsed.total_seconds()
            tracking_payload = {"publicKey": self._getPublicKey(),"language": "Python v2", "version": "1.2.5", "title": "Create-virtual-account-error","message": responseTime}
            tracking_response = requests.post(tracking_endpoint, data=json.dumps(tracking_payload))
        else:
            tracking_endpoint = self._trackingMap
            responseTime = response.elapsed.total_seconds()
            tracking_payload = {"publicKey": self._getPublicKey(),"language": "Python v2", "version": "1.2.5", "title": "Create-virtual-account","message": responseTime}
            tracking_response = requests.post(tracking_endpoint, data=json.dumps(tracking_payload))

        return self._handleCreateResponse(response, accountDetails)


import json, requests, copy
from rave_python.rave_base import RaveBase
from rave_python.rave_misc import checkIfParametersAreComplete
from rave_python.rave_exceptions import ServerError, IncompleteCardDetailsError, RecipientCreationError, RecipientStatusError

class Recipient(RaveBase):
    def __init__(self, publicKey, secretKey, production, usingEnv):
        self.headers = {
            'content-type' : 'application/json'
        }
        super(Recipient, self).__init__(publicKey, secretKey, production, usingEnv)

    def _preliminaryResponseChecks(self, response, TypeOfErrorToRaise, name):
        #check if we can get json
        try:
            responseJson = response.json()
        except:
            raise ServerError({"error": True, "name": name, "errMsg": response})

        #check for data parameter in response 
        if not responseJson.get("data", None):
            raise TypeOfErrorToRaise({"error": True, "name": name, "errMsg": responseJson.get("message", "Server is down")})

        #check for 200 response
        if not response.ok:
            errMsg = response["data"].get("message", None)
            raise TypeOfErrorToRaise({"error": True, "errMsg": errMsg})

        return responseJson

    def _handleCreateResponse(self, response, details):
        responseJson = self._preliminaryResponseChecks(response, RecipientCreationError, details["account_number"])

        if responseJson["status"] == "success":
            return {"error": False, "id": responseJson["data"].get("id", None), "data": responseJson["data"] }

        else:
            raise RecipientCreationError({"error": True, "data": responseJson["data"]})

    def _handleCardStatusRequests(self, type, endpoint, isPostRequest=False, data=None):
        #check if resposnse is a post response
        if isPostRequest:
            response = requests.post(endpoint, headers=self.headers, data=json.dumps(data))
        else:
            response = requests.get(endpoint, headers=self.headers)
        
        #check if it can be parsed to JSON
        try:
            responseJson = response.json()
        except:
            raise ServerError({"error": True, "errMsg": response.text})

        if response.ok:
            return {"error": False, "returnedData": responseJson}
        else:
            raise RecipientStatusError(type, {"error": True, "returnedData": responseJson })

    #function to create a beneficiary or transfer recipient 
    #Params: details - a dict containing currency, amount,
    def create(self, details):
        details = copy.copy(details)
        details.update({"seckey": self._getSecretKey()})
        
        requiredParameters = ["account_number", "account_bank"]
        checkIfParametersAreComplete(requiredParameters, details)

        endpoint = self._baseUrl + self._endpointMap["recipient"]["create"]
        response = requests.post(endpoint, headers=self.headers, data=json.dumps(details))
        return self._handleCreateResponse(response, details)

    def all(self):
        endpoint = self._baseUrl + self._endpointMap["recipient"]["list"] + "?seckey=" + self._getSecretKey()
        return self._handleCardStatusRequests("List", endpoint)

    def fetch(self, recipient_id):
        endpoint = self._baseUrl + self._endpointMap["recipient"]["fetch"] + "/" + str(recipient_id) + "?seckey="+self._getSecretKey()
        return self._handleCardStatusRequests("Fetch", endpoint)

    def cancel(self, recipient_id):
        if not recipient_id:
            return "recipient id was not supplied. Kindly supply one"
        endpoint = self._baseUrl + self._endpointMap["recipient"]["delete"]
        data = {
            "seckey": self._getSecretKey(),
            "id": recipient_id,
        }
        return self._handleCardStatusRequests("Cancel", endpoint, isPostRequest=True, data=data)

    


import requests, json, copy
from rave_python.rave_base import RaveBase
from rave_python.rave_misc import checkIfParametersAreComplete, generateTransactionReference, checkTransferParameters
from rave_python.rave_exceptions import InitiateTransferError, ServerError, TransferFetchError, IncompletePaymentDetailsError


class Transfer(RaveBase):
    def __init__(self, publicKey, secretKey, production, usingEnv):
        super(Transfer, self).__init__(publicKey, secretKey, production, usingEnv)
    
    def _preliminaryResponseChecks(self, response, TypeOfErrorToRaise, reference):
        # Check if we can obtain a json
        try:
            responseJson = response.json()
        except:
            raise ServerError({"error": True, "reference": reference, "errMsg": response})

        # Check if the response contains data parameter
        if not responseJson.get("data", None):
            raise TypeOfErrorToRaise({"error": True, "reference": reference, "errMsg": responseJson.get("message", "Server is down")})
        
        # Check if it is returning a 200
        if not response.ok:
            errMsg = responseJson["data"].get("message", None)
            raise TypeOfErrorToRaise({"error": True, "errMsg": errMsg})

        return responseJson


    def _handleInitiateResponse(self, response, transferDetails):
        responseJson = self._preliminaryResponseChecks(response, InitiateTransferError, transferDetails["reference"])
        
        if responseJson["status"] == "success":
            return {"error": False, "id": responseJson["data"].get("id", None), "data": responseJson["data"]}
        
        else:
            raise InitiateTransferError({"error": True, "data": responseJson["data"]})

    def _handleBulkResponse(self, response, bulkDetails):
        responseJson = self._preliminaryResponseChecks(response, InitiateTransferError, None)

        if responseJson["status"] == "success":
            return {"error": False, "status": responseJson["status"], "message":responseJson["message"], "id": responseJson["data"].get("id", None), "data": responseJson["data"]}
        else:
            raise InitiateTransferError({"error": True, "data": responseJson["data"]})

            
    def initiate(self, transferDetails):
        
        
        ## feature logic
        # Performing shallow copy of transferDetails to avoid public exposing payload with secret key
        transferDetails = copy.copy(transferDetails)
        
        # adding reference if not already included
        if not ("reference" in transferDetails):
            transferDetails.update({"reference": generateTransactionReference()})
        transferDetails.update({"seckey": self._getSecretKey()})
        
        # These are the parameters required to initiate a transfer
        requiredParameters = ["amount", "currency","beneficiary_name"]
        checkIfParametersAreComplete(requiredParameters, transferDetails)
        checkTransferParameters(requiredParameters, transferDetails)
        
        # Collating request headers
        headers = {
            'content-type': 'application/json',
        }

        endpoint = self._baseUrl + self._endpointMap["transfer"]["initiate"]
        response = requests.post(endpoint, headers=headers, data=json.dumps(transferDetails))

        if response.ok == False:
            #feature logging
            tracking_endpoint = self._trackingMap
            responseTime = response.elapsed.total_seconds()
            tracking_payload = {"publicKey": self._getPublicKey(),"language": "Python v2", "version": "1.2.5", "title": "Initiate-Transfer-error","message": responseTime}
            tracking_response = requests.post(tracking_endpoint, data=json.dumps(tracking_payload))
        else:
            tracking_endpoint = self._trackingMap
            responseTime = response.elapsed.total_seconds()
            tracking_payload = {"publicKey": self._getPublicKey(),"language": "Python v2", "version": "1.2.5", "title": "Initiate-Transfer","message": responseTime}
            tracking_response = requests.post(tracking_endpoint, data=json.dumps(tracking_payload))
        return self._handleInitiateResponse(response, transferDetails)



    def bulk(self, bulkDetails):
        
        # feature logic
        bulkDetails = copy.copy(bulkDetails)
        
        # Collating request headers
        headers = {
            'content-type': 'application/json',
        }

        bulkDetails.update({"seckey": self._getSecretKey()})
        requiredParameters = ["title", "bulk_data"]
        checkIfParametersAreComplete(requiredParameters, bulkDetails)
        checkTransferParameters(requiredParameters, bulkDetails)
        endpoint = self._baseUrl + self._endpointMap["transfer"]["bulk"]
        
        # Collating request headers
        headers = {
            'content-type': 'application/json',
        }
        response = requests.post(endpoint, headers=headers, data=json.dumps(bulkDetails))
        
        if response.ok == False:
            #feature logging
            tracking_endpoint = self._trackingMap
            responseTime = response.elapsed.total_seconds()
            tracking_payload = {"publicKey": self._getPublicKey(),"language": "Python v2", "version": "1.2.5", "title": "Initiate-Bulk-error","message": responseTime}
            tracking_response = requests.post(tracking_endpoint, data=json.dumps(tracking_payload))
        else:
            tracking_endpoint = self._trackingMap
            responseTime = response.elapsed.total_seconds()
            tracking_payload = {"publicKey": self._getPublicKey(),"language": "Python v2", "version": "1.2.5", "title": "Initiate-Bulk","message": responseTime}
            tracking_response = requests.post(tracking_endpoint, data=json.dumps(tracking_payload))

        return self._handleBulkResponse(response, bulkDetails)

    
    # This makes and handles all requests pertaining to the status of your transfer or account
    def _handleTransferStatusRequests(self, endpoint, feature_name, isPostRequest=False, data=None):
        
        # Request headers
        headers = {
            'content-type': 'application/json',
        }

        # Checks if it is a post request
        if isPostRequest:
            response = requests.post(endpoint, headers=headers, data=json.dumps(data))
        else:
            response = requests.get(endpoint, headers=headers)

        # Checks if it can be parsed to json
        try:
            responseJson = response.json()
        except:
            raise ServerError({"error": True, "errMsg": response.text })

        # Checks if it returns a 2xx code
        if response.ok:
            tracking_endpoint = self._trackingMap
            responseTime = response.elapsed.total_seconds()
            tracking_payload = {"publicKey": self._getPublicKey(),"language": "Python v2", "version": "1.2.5", "title": feature_name,"message": responseTime}
            tracking_response = requests.post(tracking_endpoint, data=json.dumps(tracking_payload))
            return {"error": False, "returnedData": responseJson}
        else:
            tracking_endpoint = self._trackingMap
            responseTime = response.elapsed.total_seconds()
            tracking_payload = {"publicKey": self._getPublicKey(),"language": "Python v2", "version": "1.2.5", "title": feature_name + "-error","message": responseTime}
            raise TransferFetchError({"error": True, "returnedData": responseJson })

    # Not elegant but supports python 2 and 3
    def fetch(self, reference=None):
        
        #feature logic
        label = "Fetch-Transfer"
        endpoint = self._baseUrl + self._endpointMap["transfer"]["fetch"] + "?seckey="+self._getSecretKey()+'&reference='+str(reference)
        return self._handleTransferStatusRequests(label, endpoint)

    def all(self):
        
        #feature logic
        label = "List-all-Transfers"
        endpoint = self._baseUrl + self._endpointMap["transfer"]["fetch"] + "?seckey="+self._getSecretKey()
        return self._handleTransferStatusRequests(label, endpoint)

    def getFee(self, currency=None):
        
        # feature logic
        label = "Get-Transfer-fee-by-Currency"
        endpoint = self._baseUrl + self._endpointMap["transfer"]["fee"] + "?seckey="+self._getSecretKey() + "&currency="+str(currency)
        return self._handleTransferStatusRequests(label, endpoint)
        
    def getBalance(self, currency):
        
        # feature logic
        label = "Get-Balance-fee-by-Currency"
        if not currency: # i made currency compulsory because if it is not assed in, an error message is returned from the server
            raise IncompletePaymentDetailsError("currency", ["currency"])
        endpoint =  self._baseUrl + self._endpointMap["transfer"]["balance"] 
        data = {
            "seckey": self._getSecretKey(),
            "currency": currency
        }

        return self._handleTransferStatusRequests(label, endpoint, data=data, isPostRequest=True)

    




name = "rave_python"
__version__ = '1.0.1'
__author__ = "Flutterwave"
__license__ = 'MIT'
__copyright__ = 'Copyright 2019. Flutterwave'


from rave_python.rave import Rave
import rave_python.rave_misc as Misc
import rave_python.rave_exceptions as RaveExceptions



import json, requests, copy
from rave_python.rave_base import RaveBase
from rave_python.rave_misc import checkIfParametersAreComplete
from rave_python.rave_exceptions import ServerError, IncompleteCardDetailsError, CardCreationError, CardStatusError

class VirtualCard(RaveBase):
    def __init__(self, publicKey, secretKey, production, usingEnv):
        self.headers = {
            'content-type' : 'application/json'
        }
        super(VirtualCard, self).__init__(publicKey, secretKey, production, usingEnv)

    def _preliminaryResponseChecks(self, response, TypeOfErrorToRaise, name):
        #check if we can get json
        try:
            responseJson = response.json()
        except:
            raise ServerError({"error": True, "name": name, "errMsg": response})

        #check for data parameter in response 
        if not responseJson.get("data", None):
            raise TypeOfErrorToRaise({"error": True, "name": name, "errMsg": responseJson.get("message", "Server is down")})

        #check for 200 response
        if not response.ok:
            errMsg = response["data"].get("message", None)
            raise TypeOfErrorToRaise({"error": True, "errMsg": errMsg})

        return responseJson

    def _handleCreateResponse(self, response, vcardDetails):
        responseJson = self._preliminaryResponseChecks(response, CardCreationError, vcardDetails["billing_name"])

        if responseJson["status"] == "success":
            return {"error": False, "id": responseJson["data"].get("id", None), "data": responseJson["data"] }

        else:
            raise CardCreationError({"error": True, "data": responseJson["data"]})

    def _handleCardStatusRequests(self, type, endpoint, feature_name, isPostRequest=False, data=None):
        #check if response is a post response
        if isPostRequest:
            response = requests.post(endpoint, headers=self.headers, data=json.dumps(data))
        else:
            response = requests.get(endpoint, headers=self.headers)
        
        #check if it can be parsed to JSON
        try:
            responseJson = response.json()
        except:
            raise ServerError({"error": True, "errMsg": response.text})

        if response.ok:
            #feature logging
            tracking_endpoint = self._trackingMap
            responseTime = response.elapsed.total_seconds()
            tracking_payload = {"publicKey": self._getPublicKey(),"language": "Python v2", "version": "1.2.5", "title": feature_name,"message": responseTime}
            tracking_response = requests.post(tracking_endpoint, data=json.dumps(tracking_payload))

            return {"error": False, "returnedData": responseJson}
        else:
            tracking_endpoint = self._trackingMap
            responseTime = response.elapsed.total_seconds()
            tracking_payload = {"publicKey": self._getPublicKey(),"language": "Python v2", "version": "1.2.5", "title": feature_name + "-error","message": responseTime}
            tracking_response = requests.post(tracking_endpoint, data=json.dumps(tracking_payload))

            raise CardStatusError(type, {"error": True, "returnedData": responseJson })


    
    #function to create a virtual card 
    #Params: vcardDetails - a dict containing currency, amount, billing_name, billing_address, billing_city, billing_state, billing_postal_code, billing_country
    def create(self, vcardDetails):
        
        #card creating logic
        vcardDetails = copy.copy(vcardDetails)
        vcardDetails.update({"seckey": self._getSecretKey()})
        requiredParameters = ["currency", "amount", "billing_name", "billing_address", "billing_city", "billing_state", "billing_postal_code", "billing_country"]
        checkIfParametersAreComplete(requiredParameters, vcardDetails)
        endpoint = self._baseUrl + self._endpointMap["virtual_card"]["create"]
        response = requests.post(endpoint, headers=self.headers, data=json.dumps(vcardDetails))
        
        #feature logging
        if response.ok == False:
            tracking_endpoint = self._trackingMap
            responseTime = response.elapsed.total_seconds()
            tracking_payload = {"publicKey": self._getPublicKey(),"language": "Python v2", "version": "1.2.5", "title": "Create-card-error", "message": responseTime}
            tracking_response = requests.post(tracking_endpoint, data=json.dumps(tracking_payload))
        else:
            tracking_endpoint = self._trackingMap
            responseTime = response.elapsed.total_seconds()
            tracking_payload = {"publicKey": self._getPublicKey(),"language": "Python v2", "version": "1.2.5", "title": "Create-card","message": responseTime}
            tracking_response = requests.post(tracking_endpoint, data=json.dumps(tracking_payload))

        return self._handleCreateResponse(response, vcardDetails)

    
    #gets all virtual cards connected to a merchant's account
    def all(self):
        
        #logic for listing all cards
        label = "List-all-cards"
        endpoint = self._baseUrl + self._endpointMap["virtual_card"]["list"] + "?seckey="+ self._getSecretKey()
        data = {"seckey": self._getSecretKey()}
        return self._handleCardStatusRequests("List", endpoint, label, isPostRequest=True, data=data)

    
    #permanently deletes a card with specified id 
    def cancel(self, card_id):
        
        if not card_id:
            return "Card id was not supplied. Kindly supply one"
        
        #card cancel feature logic
        label = "Delete-card"
        endpoint = self._baseUrl + self._endpointMap["virtual_card"]["terminate"] + str(card_id) + "/terminate"
        data = {"seckey": self._getSecretKey()}
        return self._handleCardStatusRequests("Cancel", endpoint, label, isPostRequest=True, data=data)

    
    #fetches Card details and transactions for a cars with specified id
    def get(self, card_id):
        
        #fetch card logic
        label = "Fetch-card"
        endpoint = self._baseUrl + self._endpointMap["virtual_card"]["get"]
        data = {"seckey": self._getSecretKey()}
        return self._handleCardStatusRequests("Get", endpoint, label, isPostRequest=True, data=data)

    
    #temporarily suspends the use of card
    def freeze(self, card_id):
        
        #feature logic
        label = "Block-card"
        endpoint = self._baseUrl + self._endpointMap["virtual_card"]["freeze"] + str(card_id) + "/status/block"
        data = {"seckey": self._getSecretKey()}
        return self._handleCardStatusRequests("Freeze", endpoint, label, isPostRequest=True, data=data)

    
    #reverses the freeze card operation
    def unfreeze(self, card_id):
        
        #feature logic
        label = "Unblock-card"
        endpoint = self._baseUrl + self._endpointMap["virtual_card"]["unfreeze"] + str(card_id) + "/status/unblock"
        data = {"seckey": self._getSecretKey()}
        return self._handleCardStatusRequests("Unfreeze", endpoint, label, isPostRequest=True, data=data)

    
    #funds a card with specified balance for online transactions
    def fund(self, card_id, currency, amount):
        
        #feature logic
        data = {
            "card_id": card_id,
            "amount" : amount,
            "debit_currency": currency,
            "seckey": self._getSecretKey(),
        }
        label = "Fund-card"
        endpoint = self._baseUrl + self._endpointMap["virtual_card"]["fund"]
        return self._handleCardStatusRequests("Fund", endpoint, label, isPostRequest=True, data=data)

    #withdraws funds from Virtual card. Withdrawn funds are added to Rave Balance
    def withdraw(self, card_id, amount):
        
        #feature logic
        data = {
            "card_id": card_id,
            "amount" : amount,
            "seckey": self._getSecretKey(),
        }
        label = "Withdraw-card-funds"
        endpoint = self._baseUrl + self._endpointMap["virtual_card"]["withdraw"]
        return self._handleCardStatusRequests("Fund", endpoint, label, isPostRequest=True, data=data)



import json, requests, copy
from rave_python.rave_base import RaveBase
from rave_python.rave_misc import checkIfParametersAreComplete
from rave_python.rave_exceptions import ServerError, BVNFetchError

class Verify(RaveBase):
    def __init__(self, publicKey, secretKey, production, usingEnv):
        super(Verify, self).__init__(publicKey, secretKey, production, usingEnv)

    def _preliminaryResponseChecks(self, response, TypeOfErrorToRaise, name):
        #check if we can get json
        try:
            responseJson = response.json()
        except:
            raise ServerError({"error": True, "name": name, "errMsg": response})

        #check for data parameter in response 
        if not responseJson.get("data", None):
            raise TypeOfErrorToRaise({"error": True, "name": name, "errMsg": responseJson.get("message", "Server is down")})

        #check for 200 response
        if not response.ok:
            errMsg = response["data"].get("message", None)
            raise TypeOfErrorToRaise({"error": True, "errMsg": errMsg})

        return responseJson

    # def _handleCreateResponse(self, response, details):
    #     responseJson = self._preliminaryResponseChecks(response, CardCreationError, details["billing_name"])

    #     if responseJson["status"] == "success":
    #         return {"error": False, "id": responseJson["data"].get("id", None), "data": responseJson["data"] }

    #     else:
    #         raise CardCreationError({"error": True, "data": responseJson["data"]})

    def _handleVerifyStatusRequests(self, endpoint, feature_name, isPostRequest=False, data=None):
        self.headers = {
            'content-type' : 'application/json'
        }
        #check if resposnse is a post response
        if isPostRequest:
            response = requests.post(endpoint, headers=self.headers, data=json.dumps(data))
        else:
            response = requests.get(endpoint, headers=self.headers)
        
        #check if it can be parsed to JSON
        try:
            responseJson = response.json()
        except:
            raise ServerError({"error": True, "errMsg": response.text})

        if response.ok:
            #feature logging
            tracking_endpoint = self._trackingMap
            responseTime = response.elapsed.total_seconds()
            tracking_payload = {"publicKey": self._getPublicKey(),"language": "Python v2", "version": "1.2.5", "title": feature_name,"message": responseTime}
            tracking_response = requests.post(tracking_endpoint, data=json.dumps(tracking_payload))
            return {"error": False, "returnedData": responseJson}
        else:
            tracking_endpoint = self._trackingMap
            responseTime = response.elapsed.total_seconds()
            tracking_payload = {"publicKey": self._getPublicKey(),"language": "Python v2", "version": "1.2.5", "title": feature_name + "-error", "message": responseTime}
            tracking_response = requests.post(tracking_endpoint, data=json.dumps(tracking_payload))

            raise BVNFetchError({"error": True, "returnedData": responseJson })

    def bvnVerify(self, bvn):

        # feature logic
        if not bvn:
            return "BVN was not supplied. Kindly supply one"
        feature_name = "BVN-verification"
        endpoint = self._baseUrl + self._endpointMap["bvn"]["verify"] +str(bvn)+"?seckey=" + self._getSecretKey()
        return self._handleVerifyStatusRequests(endpoint, feature_name)



from rave_python.rave_exceptions import RaveError, IncompletePaymentDetailsError, CardChargeError, TransactionVerificationError, ServerError
from rave_python.rave_payment import Payment
from rave_python.rave_misc import generateTransactionReference

class Card(Payment):
    """ This is the rave object for card transactions. It contains the following public functions:\n
        .charge -- This is for making a card charge\n
        .validate -- This is called if further action is required i.e. OTP validation\n
        .verify -- This checks the status of your transaction\n
    """
    def __init__(self, publicKey, secretKey, production, usingEnv):
        super(Card, self).__init__(publicKey, secretKey, production, usingEnv)


    # returns true if further action is required, false if it isn't    
    def _handleChargeResponse(self, response, txRef, request=None):
        """ This handles charge responses """
        res =  self._preliminaryResponseChecks(response, CardChargeError, txRef=txRef)

        responseJson = res["json"]
        flwRef = res["flwRef"]

        # Checking if there is auth url
        if responseJson["data"].get("authurl", "N/A") == "N/A":
            authUrl = None
        else:
            authUrl = responseJson["data"]["authurl"]

        # If all preliminary checks passed
        if not (responseJson["data"].get("chargeResponseCode", None) == "00"):
            # Otherwise we return that further action is required, along with the response
            suggestedAuth = responseJson["data"].get("suggested_auth", None)
            return {"error": False,  "validationRequired": True, "txRef": txRef, "flwRef": flwRef, "suggestedAuth": suggestedAuth, "authUrl": authUrl}
        else:
            return {"error": False, "status": responseJson["status"],  "validationRequired": False, "txRef": txRef, "flwRef": flwRef, "suggestedAuth": None, "authUrl": authUrl}

    
    def _handleRefundorVoidResponse(self, response, txRef, request=None):
        """ This handles charge responses """
        res =  self._preliminaryResponseChecks(response, CardChargeError, txRef=txRef)

        responseJson = res["json"]
        flwRef = responseJson["data"]["data"]["authorizeId"]

        # If all preliminary checks passed
        if not (responseJson["data"]["data"].get("responsecode", None) == "RR"):
            # Refund or Void could not be completed
            return {"error": True, "status": responseJson["status"], "message": responseJson["message"], "flwRef": flwRef}
        else:
            return {"error": False, "status": responseJson["status"], "message": responseJson["message"], "flwRef": flwRef}

    

    # This can be altered by implementing classes but this is the default behaviour
    # Returns True and the data if successful
    def _handleVerifyResponse(self, response, txRef):
        """ This handles all responses from the verify call.\n
             Parameters include:\n
            response (dict) -- This is the response Http object returned from the verify call
         """

        # Checking if there was a server error during the call (in this case html is returned instead of json)
        res =  self._preliminaryResponseChecks(response, TransactionVerificationError, txRef=txRef)
        responseJson = res["json"]

        flwRef = responseJson["data"]["flwref"]
        amount = responseJson["data"]["amount"]
        chargedamount = responseJson["data"]["chargedamount"]
        cardToken = responseJson["data"]["card"]["card_tokens"][0]["embedtoken"]
        vbvmessage = responseJson["data"]["vbvmessage"]
        chargemessage = responseJson["data"]["chargemessage"]
        chargecode = responseJson["data"]["chargecode"]
        currency = responseJson["data"]["currency"]
 
        # Check if the call returned something other than a 200
        if not response.ok:
            errMsg = responseJson["data"].get("message", "Your call failed with no response")
            raise TransactionVerificationError({"error": True, "txRef": txRef, "flwRef": flwRef, "errMsg": errMsg})
        
        # if the chargecode is not 00
        elif not (responseJson["data"].get("chargecode", None) == "00"):
            return {"error": False, "transactionComplete": False, "txRef": txRef, "flwRef":flwRef, "amount": amount, "chargedamount": chargedamount, "cardToken": cardToken, "vbvmessage": vbvmessage, "chargemessage": chargemessage, "chargecode": chargecode, "currency": currency}
        
        else:
            return {"error":False, "transactionComplete": True, "txRef": txRef, "flwRef": flwRef, "amount": amount, "chargedamount": chargedamount, "cardToken": cardToken, "vbvmessage": vbvmessage, "chargemessage": chargemessage, "chargecode": chargecode, "currency": currency}

    
    # Charge card function
    def charge(self, cardDetails, hasFailed=False, chargeWithToken=False):
        """ This is called to initiate the charge process.\n
             Parameters include:\n
            cardDetails (dict) -- This is a dictionary comprising payload parameters.\n
            hasFailed (bool) -- This indicates whether the request had previously failed for timeout handling
        """
        # setting the endpoint
        feature_name = "Initiate-Card-charge"
        if not chargeWithToken:
            endpoint = self._baseUrl + self._endpointMap["card"]["charge"]
            requiredParameters = ["cardno", "cvv", "expirymonth", "expiryyear", "amount", "email", "IP", "phonenumber", "firstname", "lastname"]
            # optionalParameters = ["phonenumber", "firstname", "lastname"]
        else: 
            if "charge_type" in cardDetails and cardDetails["charge_type"] == 'preauth':
                endpoint = self._baseUrl + self._endpointMap["preauth"]["charge"]
            else: 
                endpoint = self._baseUrl + self._endpointMap["card"]["chargeSavedCard"]

            requiredParameters = ["currency", "token", "country", "amount", "email", "txRef", "IP"]
            # optionalParameters = ["firstname", "lastname"]
            # add token to requiredParameters
            # requiredParameters.append("token")

        if not ("txRef" in cardDetails):
            cardDetails.update({"txRef":generateTransactionReference()})

        return super(Card, self).charge(feature_name, cardDetails, requiredParameters, endpoint)
    

    def validate(self, flwRef, otp):
        feature_name = "Validate-Card-charge"
        endpoint = self._baseUrl + self._endpointMap["card"]["validate"]
        return super(Card, self).validate(feature_name, flwRef, otp, endpoint)

    def verify(self, txRef):
        feature_name = "Verify-Card-charge"
        endpoint = self._baseUrl + self._endpointMap["card"]["verify"]
        return super(Card, self).verify(feature_name, txRef, endpoint)



class RaveError(Exception):
    def __init__(self, msg):
        """ This is an error pertaining to the usage of one of the functions in Rave """
        super(RaveError, self).__init__(msg)
        pass

class AccountChargeError(RaveError):
    """ Raised when account charge has failed """
    def __init__(self, err):
        self.err = err
    def __str__(self):
        return "Your account charge call failed with message: "+self.err["errMsg"]

class AccountCreationError(RaveError):
    """ Raised when creating a virtual account fails """
    def __init__(self, err):
        self.err = err
    
    def  __str__(self):
        return "Virtual account creation failed with error: " +self.err["errMsg"]

class AccountStatusError(RaveError):
    """Raised when fetching a Virtual account status"""
    def __init__(self, type, err):
        self.err = err
        self.type = type

    def __str__(self):
        return self.type +"ing account failed with error: " + self.err["errMsg"]

class AuthMethodNotSupportedError(RaveError):
    """ Raised when user requests for an auth method not currently supported by rave-python """
    def __init__(self, message):
        msg = "\n We do not currently support authMethod: \""+str(message)+"\". If you need this to be supported, please report in GitHub issues page"
        super(AuthMethodNotSupportedError, self).__init__(msg)

class BillCreationError(RaveError):
    """ Raised when creating a Bill fails """
    def __init__(self, err):
        self.err = err
    
    def  __str__(self):
        return "Bill creation failed with error: " +self.err["errMsg"]

class BillStatusError(RaveError):
    """Raised when fetching a Bill status"""
    def __init__(self, type, err):
        self.err = err
        self.type = type

    def __str__(self):
        return self.type +"ing bill failed with error: " + self.err["errMsg"]

class BVNFetchError(RaveError):
    """ Raised when fetching bvn fails """
    def __init__(self, err):
        self.err = err
    
    def __str__(self):
        return "BVN fetch failed with error: " + self.err["errMsg"]

class CardCreationError(RaveError):
    """ Raised when creating a virtual card fails """
    def __init__(self, err):
        self.err = err
    
    def  __str__(self):
        return "Virtual Card creation failed with error: " +self.err["errMsg"]

class CardChargeError(RaveError):
    """ Raised when card charge has failed """
    def __init__(self, err):
        self.err = err
    def __str__(self):
        return "Your card charge call failed with message: "+self.err["errMsg"]

class CardStatusError(RaveError):
    """Raised when fetching a Virtual Card status"""
    def __init__(self, type, err):
        self.err = err
        self.type = type

    def __str__(self):
        return self.type +"ing card failed with error: " + self.err["errMsg"]

class InitiateTransferError(RaveError):
    """ Raised when transfer initiation fails """
    def __init__(self, err):
        self.err = err
    
    def __str__(self):
        return "Transfer initiation failed with error: " + self.err["errMsg"]

class IncompleteAccountDetailsError(RaveError):
    """ Raised when details for card creation are incomplete"""
    def __init__(self, value, requiredParameters):
        msg = "\n\""+value+"\" was not defined in your dictionary. Please ensure you have supplied the following in the payload: \n "+' \n '.join(requiredParameters)
        super(IncompleteAccountDetailsError, self).__init__(msg)

class IncompleteCardDetailsError(RaveError):
    """ Raised when details for card creation are incomplete"""
    def __init__(self, value, requiredParameters):
        msg = "\n\""+value+"\" was not defined in your dictionary. Please ensure you have supplied the following in the payload: \n "+' \n '.join(requiredParameters)
        super(IncompleteCardDetailsError, self).__init__(msg)

class IncompletePaymentDetailsError(RaveError):
    """ Raised when card details are incomplete """
    def __init__(self, value, requiredParameters):
        msg =  "\n\""+value+"\" was not defined in your dictionary. Please ensure you have supplied the following in the payload: \n "+'  \n '.join(requiredParameters)
        super(IncompletePaymentDetailsError, self).__init__(msg)


class MobileChargeError(RaveError):
    """ Raised when mobile money charge has failed """
    def __init__(self, err):
        self.err = err

    def __str__(self):
        return "Your mobile money charge call failed with message: "+self.err["errMsg"]

class PlanCreationError(RaveError):
    """ Raised when creating a payment plan fails """
    def __init__(self, err):
        self.err = err

    def __str__(self):
        return "Plan Creation failed with error: " +self.err["errMsg"]

class PlanStatusError(RaveError):
    """ Raised when fetching plan fails """
    def __init__(self, type, err):
        self.err = err
        self.type = type
    
    def __str__(self):
        return self.type +"ing plan failed with error: " + self.err["errMsg"]

class PreauthCaptureError(RaveError):
    """ Raised when capturing a preauthorized transaction could not be completed """
    def __init__(self, err):
        self.err = err
        
    def __str__(self):
        return "Your preauth capture call failed with message: "+self.err["errMsg"]

class PreauthRefundVoidError(RaveError):
    """ Raised when capturing a preauthorized refund/void transaction could not be completed """
    def __init__(self, err):
        self.err = err
        
    def __str__(self):
        return "Your preauth refund/void call failed with message: "+self.err["errMsg"]

class RecipientCreationError(RaveError):
    """ Raised when creating a transfer recepient fails """
    def __init__(self, err):
        self.err = err

    def __str__(self):
        return "Recepient Creation failed with error: " +self.err["errMsg"]

class RecipientStatusError(RaveError):
    """ Raised when fetching transfer recepients fails """
    def __init__(self, type, err):
        self.err = err
        self.type = type
    
    def __str__(self):
        return self.type +"ing recepient failed with error: " + self.err["errMsg"]

class RefundError(RaveError):
    """ Raised when refund fails """
    def __init__(self, message):
        msg = "Your refund call failed with message: "+str(message)
        super(RefundError, self).__init__(msg)

class ServerError(RaveError):
    """ Raised when the server is down or when it could not process your request """
    def __init__(self, err):
        self.err = err
        
    def __str__(self):
        return " Server is down with error: " + self.err["errMsg"]

class SubaccountCreationError(RaveError):
    """ Raised when creating a payment plan fails """
    def __init__(self, err):
        self.err = err

    def __str__(self):
        return "Subaccount Creation failed with error: " +self.err["errMsg"]

class TransactionChargeError(RaveError):
    """ Raised when a transaction charge has failed """
    def __init__(self, err):
        self.err = err
    def __str__(self):
        return "Your account charge call failed with message: "+self.err["errMsg"]

class TransferFetchError(RaveError):
    """ Raised when fetching transfer fails """
    def __init__(self, err):
        self.err = err
    
    def __str__(self):
        return "Transfer fetch failed with error: " + self.err["errMsg"]

class TransactionValidationError(RaveError):
    """ Raised when validation (usually otp validation) fails """
    def __init__(self, err):
        self.err = err

    def __str__(self):
        return "Your transaction validation call failed with message: "+self.err["errMsg"]

class TransactionVerificationError(RaveError):
    """ Raised when transaction could not be verified """
    def __init__(self, err):
        self.err = err

    def __str__(self):
        return "Your transaction verification call failed with message: "+self.err["errMsg"]

class UssdChargeError(RaveError):
    """ Raised when ussd charge has failed """
    def __init__(self, err):
        self.err = err

    def __str__(self):
        return "Your ussd charge call failed with message: "+self.err["errMsg"]


from rave_python.rave_exceptions import RaveError, IncompletePaymentDetailsError, AccountChargeError, TransactionVerificationError, TransactionValidationError, ServerError
from rave_python.rave_payment import Payment
from rave_python.rave_misc import generateTransactionReference
import json, requests

class Mpesa(Payment):
    
    def __init__(self, publicKey, secretKey, production, usingEnv):
        super(Mpesa, self).__init__(publicKey, secretKey, production, usingEnv)

    # Charge mobile money function
    def charge(self, accountDetails, hasFailed=False):
        
        """ This is the mpesa charge call.\n
             Parameters include:\n
            accountDetails (dict) -- These are the parameters passed to the function for processing\n
            hasFailed (boolean) -- This is a flag to determine if the attempt had previously failed due to a timeout\n
        """
        ## feature logic
        # Setting the endpoint
        feature_name = "Initiate-Mpesa-charge"
        endpoint = self._baseUrl + self._endpointMap["account"]["charge"]
        # Adding boilerplate mpesa requirements
        accountDetails.update({"payment_type": "mpesa", "country":"KE", "is_mpesa":"1", "is_mpesa_lipa":"1", "currency":"KES"})
        # If transaction reference is not set 
        if not ("txRef" in accountDetails):
            accountDetails.update({"txRef": generateTransactionReference()})
        # If order reference is not set
        if not ("orderRef" in accountDetails):
            accountDetails.update({"orderRef": generateTransactionReference()})

        # Checking for required account components
        requiredParameters = ["amount", "email", "phonenumber", "IP"]
        res = super(Mpesa, self).charge(feature_name, accountDetails, requiredParameters, endpoint, isMpesa=True)
        return res

    def verify(self, txRef):
        feature_name = "Verify-Mpesa_charge"
        endpoint = self._baseUrl + self._endpointMap["account"]["verify"]
        return super(Mpesa, self).verify(feature_name, txRef, endpoint)



from rave_python.rave_payment import Payment
from rave_python.rave_misc import generateTransactionReference
import json, requests

class UGMobile(Payment):
    
    def __init__(self, publicKey, secretKey, production, usingEnv):
        super(UGMobile, self).__init__(publicKey, secretKey, production, usingEnv)

    # Charge mobile money function
    def charge(self, accountDetails, hasFailed=False):
        
        """ This is the ghMobile charge call.
             Parameters include:\n
            accountDetails (dict) -- These are the parameters passed to the function for processing\n
            hasFailed (boolean) -- This is a flag to determine if the attempt had previously failed due to a timeout\n
        """
        
        ## feature logic
        feature_name = "Initiate-Uganda-mobile-money-charge"
        endpoint = self._baseUrl + self._endpointMap["account"]["charge"]
        # It is faster to add boilerplate than to check if each one is present
        accountDetails.update({"payment_type": "mobilemoneyuganda", "country":"NG", "is_mobile_money_ug":"1", "currency":"UGX", "network": "UGX"})
        # If transaction reference is not set 
        if not ("txRef" in accountDetails):
            accountDetails.update({"txRef": generateTransactionReference()})
        # If order reference is not set
        if not ("orderRef" in accountDetails):
            accountDetails.update({"orderRef": generateTransactionReference()})
        # Checking for required account components
        requiredParameters = ["amount", "email", "phonenumber", "network", "IP"]
        return super(UGMobile, self).charge(feature_name, accountDetails, requiredParameters, endpoint)




import requests, json, copy
from rave_python.rave_base import RaveBase
from rave_python.rave_misc import checkIfParametersAreComplete, generateTransactionReference
from rave_python.rave_exceptions import  ServerError, IncompletePaymentDetailsError, PlanCreationError, PlanStatusError

class PaymentPlan(RaveBase) :
    def __init__(self, publicKey, secretKey, production, usingEnv):
        self.headers = {
            'content-type': 'application/json'
        }
        super(PaymentPlan, self).__init__(publicKey, secretKey, production, usingEnv)
    
    def _preliminaryResponseChecks(self, response, TypeOfErrorToRaise, name):
        # Check if we can obtain a json
        
        try:
            responseJson = response.json()
        except:
            raise ServerError({"error": True, "name": name, "errMsg": response})

        # Check if the response contains data parameter
        if not responseJson.get("data", None):
            raise TypeOfErrorToRaise({"error": True, "name": name, "errMsg": responseJson.get("message", "Server is down")})
        
        # Check if it is returning a 200
        if not response.ok:
            errMsg = responseJson["data"].get("message", None)
            raise TypeOfErrorToRaise({"error": True, "errMsg": errMsg})
        
        return responseJson
    
    def _handleCreateResponse(self, response, planDetails):
        responseJson = self._preliminaryResponseChecks(response, PlanCreationError, planDetails["name"])
        
        if responseJson["status"] == "success":
            return {"error": False, "id": responseJson["data"].get("id", None), "data": responseJson["data"]}
        
        else:
            raise PlanCreationError({"error": True, "data": responseJson["data"]})

    # This makes and handles all requests pertaining to the status of your payment plans
    def _handlePlanStatusRequests(self, type, endpoint, isPostRequest=False, data=None):

        # Checks if it is a post request
        if isPostRequest:
            response = requests.post(endpoint, headers=self.headers, data=json.dumps(data))
        else:
            response = requests.get(endpoint, headers=self.headers)
        # Checks if it can be parsed to json
        try:
            responseJson = response.json()
        except:
            raise ServerError({"error": True, "errMsg": response.text })

        # Checks if it returns a 2xx code
        if response.ok:
            return {"error": False, "returnedData": responseJson}
        else:
            raise PlanStatusError(type, {"error": True, "returnedData": responseJson })
    
    #function to create a payment plan
    #Params: planDetails - a dict containing amount, name, interval, duration
    #if duration is not passed, any subscribed customer will be charged #indefinitely
    def create(self, planDetails):
        # Performing shallow copy of planDetails to avoid public exposing payload with secret key
        planDetails = copy.copy(planDetails)
        planDetails.update({"seckey": self._getSecretKey()})

        requiredParameters = ["amount", "name", "interval"]
        checkIfParametersAreComplete(requiredParameters, planDetails)

        endpoint = self._baseUrl + self._endpointMap["payment_plan"]["create"]
        response = requests.post(endpoint, headers=self.headers, data=json.dumps(planDetails))
        return self._handleCreateResponse(response, planDetails)

    #gets all payment plans connected to a merchant's account
    def all(self):
        endpoint = self._baseUrl + self._endpointMap["payment_plan"]["list"] + "?seckey="+self._getSecretKey()
        return self._handlePlanStatusRequests("List", endpoint)
    
    def fetch(self, plan_id=None, plan_name=None):
        if plan_id:
            endpoint = self._baseUrl + self._endpointMap["payment_plan"]["fetch"] + "?seckey="+self._getSecretKey() + "&id="+str(plan_id)
        elif plan_name:
            endpoint = self._baseUrl + self._endpointMap["payment_plan"]["fetch"] + "?seckey="+self._getSecretKey() + "&1="+plan_name
        else:
            return "You must pass either plan id or plan name in order to fetch a plan's details"
        return self._handlePlanStatusRequests("Fetch", endpoint)
    
    def cancelPlan(self, plan_id):
        if not id:
            return "Plan id was not supplied. Kindly supply one"
        endpoint = self._baseUrl + self._endpointMap["payment_plan"]["cancel"] + str(plan_id) + "/cancel"
        data = {"seckey": self._getSecretKey()}
        return self._handlePlanStatusRequests("Cancel", endpoint, isPostRequest=True, data=data)
    
    # edits a payment plan
    # Params
    # id: payment plan id *required
    # newData: dict that contains the information to be updated i.e name and status-cancelled/active
    def edit(self, plan_id, newData={}):
        if not id:
            return "Plan id was not supplied. Kindly supply one"
        endpoint = self._baseUrl + self._endpointMap["payment_plan"]["edit"] + str(plan_id) + "/edit"
        data = {"seckey": self._getSecretKey(), "name": newData.get("name", None), "status": newData.get("status", None)}
        return self._handlePlanStatusRequests("Edit", endpoint, isPostRequest=True, data=data)



""" Miscellaneous helper functions """
import time
from rave_python.rave_exceptions import IncompletePaymentDetailsError, AuthMethodNotSupportedError
# Helper function to generate unique transaction reference
def generateTransactionReference(merchantId=None):
    """ This is a helper function for generating unique transaction  references.\n
         Parameters include:\n
        merchantId (string) -- (optional) You can specify a merchant id to start references e.g. merchantId-12345678
    """
    rawTime = round(time.time() * 1000)
    timestamp = int(rawTime)
    if merchantId:
        return merchantId+"-"+str(timestamp)
    else:
        return "MC-"+str(timestamp)


def checkTransferParameters(requiredParameters, paymentDetails):
    # Transfer specific meta parameters
    requiredTransferMetaParams = ['AccountNumber','RoutingNumber', 'BankName', 'BeneficiaryName','BeneficiaryAddress', 'BeneficiaryCountry']
    excludedCurrencies = ["NGN", "GHS", "KES", "UGX", "TZS"]
    #end Transfer specific meta parameters

    # International transfer check block
    if "bulk_data" in requiredParameters:
        for i in paymentDetails["bulk_data"]:
            if "debit_currency" not in i:
                if i["Currency"] not in excludedCurrencies:
                    if "meta" in i:
                        for j in requiredTransferMetaParams:
                            if j not in i["meta"][0]:
                                raise IncompletePaymentDetailsError(i, requiredTransferMetaParams)
                    else:
                        raise IncompletePaymentDetailsError("meta", requiredParameters)
    else:
        if "debit_currency" not in paymentDetails:
            if paymentDetails["currency"] not in excludedCurrencies:
                if "meta" in paymentDetails:
                    for i in requiredTransferMetaParams:
                        if i not in paymentDetails["meta"][0]:
                            raise IncompletePaymentDetailsError(i, requiredTransferMetaParams)
                else:
                    raise IncompletePaymentDetailsError("meta", requiredParameters)

    #end international transfer check block

# If parameters are complete, returns true. If not returns false with parameter missing
def checkIfParametersAreComplete(requiredParameters, paymentDetails):
    """ This returns true/false depending on if the paymentDetails match the required parameters """
    for i in requiredParameters:
        if i not in paymentDetails:
            raise IncompletePaymentDetailsError(i, requiredParameters)
    return True, None

def getTypeOfArgsRequired(suggestedAuth):
    """ This is used to get the type of argument needed to complete your charge call.\n
            Parameters include:\n
        suggestedAuth (dict) -- This is the action returned from the charge call\n

        Returns:\n
        pin -- This means that the updatePayload call requires a pin. Pin is passed as a string argument to updatePayload\n
        address -- This means that the updatePayload call requires an address dict. The dict must contain "billingzip", "billingcity", "billingaddress", "billingstate", "billingcountry".
    """
    keywordMap = {"PIN": "pin", "AVS_VBVSECURECODE": "address", "NOAUTH_INTERNATIONAL": "address", "AVS_NOAUTH": "address"}
    # Checks if the auth method passed is included in keywordMapping i.e. if it is supported
    if not keywordMap.get(suggestedAuth, None):
        raise AuthMethodNotSupportedError(suggestedAuth)
    
    return keywordMap[suggestedAuth]

# Update payload
def updatePayload(suggestedAuth, payload, **kwargs): 
    """ This is used to update the payload of your request upon a charge that requires more parameters. It maintains the transaction refs and all the original parameters of the request.\n
            Parameters include:\n
        suggestedAuth (dict) -- This is what is returned from the charge call\n
        payload (dict) -- This is the original payload\n
        \n
        ## This updates payload directly
    """ 

    # Sets the keyword to check for in kwargs (it maps the suggestedAuth to keywords)
    keyword = getTypeOfArgsRequired(suggestedAuth)

    # Checks

    # 1) Checks if keyword is present in kwargs
    if not kwargs.get(keyword, None):
        # Had to split variable assignment and raising ValueError because of error message python displayed
        errorMsg = "Please provide the appropriate argument for the auth method. For {}, we require a \"{}\" argument.".format(suggestedAuth["suggested_auth"], keyword)
        raise ValueError(errorMsg)

    # 2) If keyword is address, checks if all required address parameters are present
    if keyword == "address":
        requiredAddressParameters = ["billingzip", "billingcity", "billingaddress", "billingstate", "billingcountry"]
        checkIfParametersAreComplete(requiredAddressParameters, kwargs[keyword])
        
    # All checks passed

    # Add items to payload
    # If the argument is a dictionary, we add the argument as is
    if isinstance(kwargs[keyword], dict):
        payload.update(kwargs[keyword])

    # If it's not we add it manually
    else:
        payload.update({"suggested_auth": suggestedAuth})
        payload.update({keyword: kwargs[keyword]})
            
    


import os, hashlib, warnings, requests, json
from rave_python.rave_exceptions import ServerError, RefundError
import base64
from Crypto.Cipher import DES3
class RaveBase(object):
    """ This is the core of the implementation. It contains the encryption and initialization functions. It also contains all direct rave functions that require publicKey or secretKey (refund) """
    def __init__(self, publicKey=None, secretKey=None, production=False, usingEnv=True):

        # config variables (protected)
        self._baseUrlMap = ["https://ravesandboxapi.flutterwave.com/", "https://api.ravepay.co/"]
        self._trackingMap = "https://kgelfdz7mf.execute-api.us-east-1.amazonaws.com/staging/sendevent"
        self._endpointMap = {
            "bills": {
                "create":"v2/services/confluence",
            },
            "bvn": {
                "verify": "v2/kyc/bvn/",
            },
            "card": {
                "charge": "flwv3-pug/getpaidx/api/charge",
                "validate": "flwv3-pug/getpaidx/api/validatecharge",
                "verify": "flwv3-pug/getpaidx/api/v2/verify",
                "chargeSavedCard": "flwv3-pug/getpaidx/api/tokenized/charge",
            },
            "ebills": {
                "create": "flwv3-pug/getpaidx/api/ebills/generateorder/",
                "update": "flwv3-pug/getpaidx/api/ebills/update/",
            },
            "preauth": {
                "charge": "flwv3-pug/getpaidx/api/tokenized/preauth_charge",
                "capture": "flwv3-pug/getpaidx/api/capture",
                "refundorvoid": "flwv3-pug/getpaidx/api/refundorvoid"
            },
            "account": {
                "charge": "flwv3-pug/getpaidx/api/charge",
                "validate": "flwv3-pug/getpaidx/api/validate",
                "verify": "flwv3-pug/getpaidx/api/v2/verify"
            },
            "payment_plan": {
                "create": "v2/gpx/paymentplans/create",
                "fetch": "v2/gpx/paymentplans/query",
                "list": "v2/gpx/paymentplans/query",
                "cancel": "v2/gpx/paymentplans/",
                "edit" :  "v2/gpx/paymentplans/"
            },
            "subscriptions": {
                "fetch": "v2/gpx/subscriptions/query",
                "list": "v2/gpx/subscriptions/query",
                "cancel": "v2/gpx/subscriptions/",
                "activate" : "v2/gpx/subscriptions/"
            },
            "settlements": {
                "list": "v2/merchant/settlements",
                "fetch": "v2/merchant/settlements/",
            },
            "subaccount": {
                "create": "v2/gpx/subaccounts/create",
                "list": "v2/gpx/subaccounts/",
                "fetch": "v2/gpx/subaccounts/get",
                "update": "v2/gpx/subaccounts/edit",
                "delete": "v2/gpx/subaccounts/delete",
            },
            "transfer": {
                "initiate": "v2/gpx/transfers/create",
                "bulk": "v2/gpx/transfers/create_bulk",
                "fetch": "v2/gpx/transfers",
                "fee": "v2/gpx/transfers/fee",
                "balance": "v2/gpx/balance",
                "accountVerification": "flwv3-pug/getpaidx/api/resolve_account"
            },
            "recipient":{
                "create": "v2/gpx/transfers/beneficiaries/create",
                "list": "v2/gpx/transfers/beneficiaries",
                "fetch": "v2/gpx/transfers/beneficiaries",
                "delete": "v2/gpx/transfers/beneficiaries/delete",
            },
            "virtual_card": {
                "create": "v2/services/virtualcards/new",
                "list": "v2/services/virtualcards/search",
                "get": "v2/services/virtualcards/get",
                "terminate": "v2/services/virtualcards/",
                "fund": "v2/services/virtualcards/fund",
                "transactions": "v2/services/virtualcards/",
                "withdraw": "v2/services/virtualcards/withdraw",
                "freeze": "v2/services/virtualcards/",
                "unfreeze": "v2/services/virtualcards/",
            },
            "virtual_account":{
                "create" : "v2/banktransfers/accountnumbers",
            },
            "verify": "flwv3-pug/getpaidx/api/v2/verify",
            "refund": "gpx/merchant/transactions/refund"
            
        }
        

        # Setting up public and private keys (private)
        # If we are using environment variables to store secretKey
        if(usingEnv):  
            self.__publicKey = publicKey
            self.__secretKey = os.getenv("RAVE_SECRET_KEY", None)

            if (not self.__publicKey) or (not self.__secretKey):
                raise ValueError("Please set your RAVE_SECRET_KEY environment variable. Otherwise, pass publicKey and secretKey as arguments and set usingEnv to false")

        # If we are not using environment variables
        else:
            if (not publicKey) or (not secretKey):
                raise ValueError("\n Please provide as arguments your publicKey and secretKey. \n It is advised however that you provide secret key as an environment variables. \n To do this, remove the usingEnv flag and save your keys as environment variables, RAVE_PUBLIC_KEY and RAVE_SECRET_KEY")
    
            else:
                self.__publicKey = publicKey
                self.__secretKey = secretKey

                # Raise warning about not using environment variables
                warnings.warn("Though you can use the usingEnv flag to pass secretKey as an argument, it is advised to store it in an environment variable, especially in production.", SyntaxWarning)

        # Setting instance variables
        # 
        # production/non-production variables (protected)
        self._isProduction = production 
        self._baseUrl = self._baseUrlMap[production]

        # encryption key (protected)
        self._encryptionKey = self.__getEncryptionKey()

    

    # This generates the encryption key (private)
    def __getEncryptionKey(self):
        """ This generates the encryption key """
        if(self.__secretKey):
            hashedseckey = hashlib.md5(self.__secretKey.encode("utf-8")).hexdigest()
            hashedseckeylast12 = hashedseckey[-12:]
            seckeyadjusted = self.__secretKey.replace('FLWSECK-', '')
            seckeyadjustedfirst12 = seckeyadjusted[:12]
            key = seckeyadjustedfirst12 + hashedseckeylast12
            return key

        raise ValueError("Please initialize RavePay")
    
    # This returns the public key
    def _getPublicKey(self):
        return self.__publicKey
    
    # This returns the secret key
    def _getSecretKey(self):
        return self.__secretKey

    # This encrypts text
    def _encrypt(self, plainText):
        """ This is the encryption function.\n
             Parameters include:\n 
            plainText (string) -- This is the text you wish to encrypt
        """
        blockSize = 8
        padDiff = blockSize - (len(plainText) % blockSize)
        key = self.__getEncryptionKey()
        cipher = DES3.new(key, DES3.MODE_ECB)
        plainText = "{}{}".format(plainText, "".join(chr(padDiff) * padDiff))
        # cipher.encrypt - the C function that powers this doesn't accept plain string, rather it accepts byte strings, hence the need for the conversion below
        test = plainText.encode('utf-8')
        encrypted = base64.b64encode(cipher.encrypt(test)).decode("utf-8")
        return encrypted
        





    


import requests, json, copy
from rave_python.rave_base import RaveBase
from rave_python.rave_misc import checkIfParametersAreComplete, generateTransactionReference
from rave_python.rave_exceptions import  ServerError, IncompletePaymentDetailsError, SubaccountCreationError, PlanStatusError

class SubAccount(RaveBase) :
    def __init__(self, publicKey, secretKey, production, usingEnv):
        self.headers = {
            'content-type': 'application/json'
        }
        super(SubAccount, self).__init__(publicKey, secretKey, production, usingEnv)
    
    def _preliminaryResponseChecks(self, response, TypeOfErrorToRaise, name):
        # Check if we can obtain a json
        try:
            responseJson = response.json()
        except:
            raise ServerError({"error": True, "name": name, "errMsg": response})

        # Check if the response contains data parameter
        if not responseJson.get("data", None):
            raise TypeOfErrorToRaise({"error": True, "name": name, "errMsg": responseJson.get("message", "Server is down")})
        
        # Check if it is returning a 200
        if not response.ok:
            errMsg = responseJson["data"].get("message", None)
            raise TypeOfErrorToRaise({"error": True, "errMsg": errMsg})
        
        return responseJson
    
    def _handleCreateResponse(self, response, accountDetails):
        responseJson = self._preliminaryResponseChecks(response, SubaccountCreationError, accountDetails["business_email"])
        
        if responseJson["status"] == "success":
            return {"error": False, "id": responseJson["data"].get("id", None), "data": responseJson["data"]}
        
        else:
            raise SubaccountCreationError({"error": True, "data": responseJson["data"]})

    # This makes and handles all requests pertaining to the status of your payment plans
    def _handleAccountStatusRequests(self, type, endpoint, isPostRequest=False, data=None):

        # Checks if it is a post request
        if isPostRequest:
            response = requests.post(endpoint, headers=self.headers, data=json.dumps(data))
        else:
            response = requests.get(endpoint, headers=self.headers)

        # Checks if it can be parsed to json
        try:
            responseJson = response.json()
        except:
            raise ServerError({"error": True, "errMsg": response.text })

        # Checks if it returns a 2xx code
        if response.ok:
            return {"error": False, "returnedData": responseJson}
        else:
            raise PlanStatusError(type, {"error": True, "returnedData": responseJson })
    
    #function to create a payment plan
    #Params: accountDetails - a dict containing account_bank, account_number, business_name, business_email, business_contact, business_contact_mobile, business_mobile, split_type, split_value
    #if duration is not passed, any subscribed customer will be charged #indefinitely
    def create(self, accountDetails):
        # Performing shallow copy of planDetails to avoid public exposing payload with secret key
        accountDetails = copy.copy(accountDetails)
        accountDetails.update({"seckey": self._getSecretKey()})
        requiredParameters = ["account_bank", "account_number", "business_name", "business_email", "business_contact", "business_contact_mobile", "business_mobile", "split_type", "split_value"]
        checkIfParametersAreComplete(requiredParameters, accountDetails)

        endpoint = self._baseUrl + self._endpointMap["subaccount"]["create"]
        response = requests.post(endpoint, headers=self.headers, data=json.dumps(accountDetails))
        return self._handleCreateResponse(response, accountDetails)

    #gets all subaccounts connected to a merchant's account
    def all(self):
        endpoint = self._baseUrl + self._endpointMap["subaccount"]["list"] + "?seckey="+self._getSecretKey()
        return self._handleAccountStatusRequests("List", endpoint)
    
    def fetch(self, subaccount_id):
        if not subaccount_id:
            return "No subaccount id supplied. Kindly pass one in"
        endpoint = self._baseUrl + self._endpointMap["subaccount"]["fetch"] + "/" +str(subaccount_id) + "?seckey="+self._getSecretKey()
        return self._handleAccountStatusRequests("Fetch", endpoint)

    def edit(self, Subaccount_id, newData={}):
        if not id:
            return "Plan id was not supplied. Kindly supply one"
        endpoint = self._baseUrl + self._endpointMap["subaccount"]["update"]
        data = {
            "seckey": self._getSecretKey(), 
            "account_number": newData.get("account_number", None), 
            "account_bank": newData.get("account_bank", None),
            "business_name": newData.get("business_name", None),
            "business_email": newData.get("business_email", None),
            "split_type": newData.get("split_type", None),
            "split_value": newData.get("split_value", None),
            }
        return self._handlePlanStatusRequests("Edit", endpoint, isPostRequest=True, data=data)
    
    def cancel(self, subaccount_id):
        if not subbacount_id:
            return "Subaccount id was not supplied. Kindly supply one"
        endpoint = self._baseUrl + self._endpointMap["subaccount"]["delete"]
        data = {
            "seckey": self._getSecretKey(),
            "id": subaccount_id,
            }
        return self._handlePlanStatusRequests("Cancel", endpoint, isPostRequest=True, data=data)
    



import json, requests, copy
from rave_python.rave_base import RaveBase
from rave_python.rave_misc import checkIfParametersAreComplete
from rave_python.rave_exceptions import ServerError, IncompleteCardDetailsError, BillCreationError, BillStatusError

class Bills(RaveBase):
    def __init__(self, publicKey, secretKey, production, usingEnv):
        self.headers = {
            'content-type' : 'application/json'
        }
        super(Bills, self).__init__(publicKey, secretKey, production, usingEnv)

    def _preliminaryResponseChecks(self, response, TypeOfErrorToRaise, name):
        #check if we can get json
        try:
            responseJson = response.json()
        except:
            raise ServerError({"error": True, "name": name, "errMsg": response})

        #check for data parameter in response 
        if not responseJson.get("data", None):
            raise TypeOfErrorToRaise({"error": True, "name": name, "errMsg": responseJson.get("message", "Server is down")})

        #check for 200 response
        if not response.ok:
            errMsg = response["data"].get("message", None)
            raise TypeOfErrorToRaise({"error": True, "errMsg": errMsg})

        return responseJson

    def _handleCreateResponse(self, response, details):
        responseJson = self._preliminaryResponseChecks(response, BillCreationError, details["service"])

        if responseJson["status"] == "success":
            return {"error": False, "id": responseJson["data"].get("id", None), "data": responseJson["data"] }

        else:
            raise BillCreationError({"error": True, "data": responseJson["data"]})


    #function to create a Bill
    #Params: details - a dict containing service, service_method, service_version, service_channel and service_payload
    def create(self, details):
        
        # Performing shallow copy of planDetails to avoid public exposing payload with secret key
        details = copy.copy(details)
        details.update({"seckey": self._getSecretKey()})
        requiredParameters = ["service", "service_method", "service_version", "service_channel"]
        checkIfParametersAreComplete(requiredParameters, details)
        endpoint = self._baseUrl + self._endpointMap["bills"]["create"]
        response = requests.post(endpoint, headers=self.headers, data=json.dumps(details))

        # feature logging
        if response.ok == False:
            tracking_endpoint = self._trackingMap
            responseTime = response.elapsed.total_seconds()
            tracking_payload = {"publicKey": self._getPublicKey(),"language": "Python v2", "version": "1.2.5", "title": "Create-Bills-error", "message": responseTime}
            tracking_response = requests.post(tracking_endpoint, data=json.dumps(tracking_payload))
        else:
            tracking_endpoint = self._trackingMap
            responseTime = response.elapsed.total_seconds()
            tracking_payload = {"publicKey": self._getPublicKey(),"language": "Python v2", "version": "1.2.5", "title": "Create-Bills", "message": responseTime}
            tracking_response = requests.post(tracking_endpoint, data=json.dumps(tracking_payload))

        return self._handleCreateResponse(response, details)



from rave_python.rave_account import Account
from rave_python.rave_bills import Bills
from rave_python.rave_card import Card
from rave_python.rave_ebills import Ebills
from rave_python.rave_francophone import Francophone
from rave_python.rave_ghmobile import GhMobile
from rave_python.rave_mpesa import Mpesa
from rave_python.rave_paymentplan import PaymentPlan
from rave_python.rave_preauth import Preauth
from rave_python.rave_base import RaveBase
from rave_python.rave_recipient import Recipient
from rave_python.rave_rwmobile import RWMobile
from rave_python.rave_settlement import Settlement
from rave_python.rave_subaccounts import SubAccount
from rave_python.rave_subscription import Subscriptions
from rave_python.rave_transfer import Transfer
from rave_python.rave_ugmobile import UGMobile
from rave_python.rave_ussd import Ussd
from rave_python.rave_verify import Verify
from rave_python.rave_virtualaccount import VirtualAccount
from rave_python.rave_virtualcard import VirtualCard
from rave_python.rave_zbmobile import ZBMobile


class Rave:
    
    def __init__(self, publicKey, secretKey, production=False, usingEnv=True):
        """ This is main organizing object. It contains the following:\n
            rave.Account -- For bank account transactions\n
            rave.Bills -- For Bills payments\n
            rave.Card -- For card transactions\n
            rave.Francophone -- For West African Francophone mobile money transactions\n
            rave.GhMobile -- For Ghana mobile money transactions\n
            rave.Mpesa -- For mpesa transactions\n
            rave.PaymentPlan -- For payment plan creation and operation\n
            rave.Preauth -- For preauthorized transactions\n
            rave.RWMobile -- For Rwanda mobile money transactions\n
            rave.Settlement -- For settled transactions\n
            rave.SubAccount -- For creation of subaccounts for split payment operations\n
            rave.Transfer -- For Payouts and transfers\n
            rave.UGMobile -- For Uganda mobile money transactions\n
            rave.Ussd -- For ussd transactions\n
            rave.VirtualAccount -- For virtual account transactions\n
            rave.VirtualCard -- For virtual card transactions\n 
            rave.ZBMobile -- For Zambia mobile money transactions\n
            
        """

        classes = (
            Account, Bills, Card, Ebills, Francophone, GhMobile, Mpesa, PaymentPlan, Preauth, Recipient, RWMobile, Settlement, SubAccount, Subscriptions, Transfer, UGMobile, Ussd, Verify, VirtualAccount, VirtualCard, ZBMobile
        )

        for _class in classes:
            attr = _class(publicKey, secretKey, production, usingEnv)
            setattr(self, _class.__name__, attr)
        
      



import requests
import json
from rave_python.rave_exceptions import ServerError, TransactionVerificationError, PreauthCaptureError, PreauthRefundVoidError
from rave_python.rave_card import Card
from rave_python.rave_misc import generateTransactionReference

class Preauth(Card):
    """ This is the rave object for preauthorized transactions. It contains the following public functions:\n
        .charge -- This is for preauthorising a specified amount\n
        .capture -- This is for capturing a preauthorized amount\n
        .validate -- This is called if further action is required i.e. OTP validation\n
        .verify -- This checks the status of your transaction\n
    """

    def __init__(self, publicKey, secretKey, production, usingEnv):
        super(Preauth, self).__init__(publicKey, secretKey, production, usingEnv)

    # Initiate preauth
    def charge(self, cardDetails, chargeWithToken=False, hasFailed=False):
        """ This is called to initiate the preauth process.\n
             Parameters include:\n
            cardDetails (dict) -- This is a dictionary comprising payload parameters.\n
            hasFailed (bool) -- This indicates whether the request had previously failed for timeout handling
        """

        # Add the charge_type
        cardDetails.update({"charge_type":"preauth"})
        if not chargeWithToken:
            return super(Preauth, self).charge(cardDetails, chargeWithToken=False)
        else:
            return super(Preauth, self).charge(cardDetails, chargeWithToken=True)
    

    # capture payment
    def capture(self, flwRef ):
        """ This is called to complete the transaction.\n
             Parameters include:
            flwRef (string) -- This is the flutterwave reference you receive from action["flwRef"]
        """
        payload = {
            "SECKEY": self._getSecretKey(),
            "flwRef": flwRef
        }
        headers ={
            "Content-Type":"application/json"
        }
        endpoint = self._baseUrl + self._endpointMap["preauth"]["capture"]
        response = requests.post(endpoint, headers=headers, data=json.dumps(payload))
        return self._handleCaptureResponse(response, '')
    

    def void(self, flwRef):
        """ This is called to void a transaction.\n 
             Parameters include:\n
            flwRef (string) -- This is the flutterwave reference you receive from action["flwRef"]\n
        """
        payload = {
            "SECKEY": self._getSecretKey(),
            "ref": flwRef,
            "action":"void"
        }
        headers ={
            "Content-Type":"application/json"
        }
        endpoint = self._baseUrl + self._endpointMap["preauth"]["refundorvoid"]
        response = requests.post(endpoint, headers=headers, data=json.dumps(payload))
        return self._handleRefundorVoidResponse(response, endpoint)
    
    
    def refund(self, flwRef, amount=None):
        """ This is called to refund the transaction.\n
             Parameters include:\n
            flwRef (string) -- This is the flutterwave reference you receive from action["flwRef"]\n
            amount (Number) -- (optional) This is called if you want a partial refund
        """
        payload = {
            "SECKEY": self._getSecretKey(),
            "ref": flwRef,
            "action":"refund"
        }
        if amount:
            payload["amount"] = amount

        headers ={
            "Content-Type":"application/json"
        }
        endpoint = self._baseUrl + self._endpointMap["preauth"]["refundorvoid"]
        response = requests.post(endpoint, headers=headers, data=json.dumps(payload))
        return self._handleRefundorVoidResponse(response, endpoint)



from rave_python.rave_exceptions import UssdChargeError, ServerError

from rave_python.rave_payment import Payment
from rave_python.rave_misc import generateTransactionReference
import json

class Ussd(Payment):
    def __init__(self, publicKey, secretKey, production, usingEnv):
        """ This is the rave object for ussd transactions. It contains the following public functions:\n
        .charge -- This is for making a ussd charge\n
        .verify -- This checks the status of your transaction\n
        """
        super(Ussd, self).__init__(publicKey, secretKey, production, usingEnv)
        

    def _handleChargeResponse(self, response, txRef, request):
        bankList = {"gtb": "058", "zenith": "057"}
        gtbResponseText = "To complete this transaction, please dial *737*50*charged_amount*159#"
        # This checks if we can parse the json successfully
        
        res =  self._preliminaryResponseChecks(response, UssdChargeError, txRef=txRef)

        responseJson = res["json"]
        flwRef = res["flwRef"]

        # Charge response code of 00 means successful, 02 means failed. Here we check if the code is not 00
        if not (responseJson["data"].get("chargeResponseCode", None) == "00"):
            # If it is we return that further action is required
            # If it is a gtbank account
            if request["accountbank"] == bankList["gtb"]:
                return {"error": False, "validationRequired": True, "txRef": txRef, "flwRef": flwRef, "validationInstruction": gtbResponseText}
            else:
                return {"error": False, "validationRequired": True, "txRef": txRef, "flwRef": flwRef, "validationInstruction": responseJson["data"].get("validateInstructions", None)}

        # If a charge is successful, we return that further action is not required, along with the response
        else:
            return {"error": False, "validationRequired": False, "txRef": txRef, "flwRef": flwRef, "validationInstruction": None}


    # Charge ussd function
    def charge(self, ussdDetails, hasFailed=False):
        """ This is used to charge through ussd.\n
             Parameters are:\n
            ussdDetails (dict) -- This is a dictionary comprising payload parameters.\n
            hasFailed (bool) -- This indicates whether the request had previously failed for timeout handling
        """

        endpoint = self._baseUrl + self._endpointMap["account"]["charge"]

        # Add boilerplate ussd code
        ussdDetails.update({"is_ussd": "1", "payment_type": "ussd"})
        # if transaction reference is not present, generate
        if not ("txRef" in ussdDetails):
            ussdDetails.update({"txRef": generateTransactionReference()})
        if not ("orderRef" in ussdDetails):
            ussdDetails.update({"orderRef": generateTransactionReference()})
        # Checking for required ussd components (not checking for payment_type, is_ussd, txRef or orderRef again to increase efficiency)
        requiredParameters = ["accountbank", "accountnumber", "amount", "email", "phonenumber", "IP"]

        # Should return request is a less efficient call but it is required here because we need bank code in _handleResponses
        return super(Ussd, self).charge(ussdDetails, requiredParameters, endpoint, shouldReturnRequest=True)







import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="rave_python",
    version="1.2.7",
    author="Flutterwave",
    author_email="developers@flutterwavego.com",
    description="Official Rave Python Wrapper By Flutterwave",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/Flutterwave/rave-python",
    license="MIT",
    packages=setuptools.find_packages(),
    python_requires=">=2.7, !=3.0.*, !=3.1.*, !=3.2.*, !=3.3.*, !=3.4.*",
    classifiers=(
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ),
    install_requires = [
        'pycryptodome',
        'requests'
    ]
)


import requests, json, copy
from rave_python.rave_base import RaveBase
from rave_python.rave_exceptions import RaveError, IncompletePaymentDetailsError, AuthMethodNotSupportedError, TransactionChargeError, TransactionVerificationError, TransactionValidationError, ServerError, RefundError, PreauthCaptureError
from rave_python.rave_misc import checkIfParametersAreComplete

response_object = {
    "error": False,
    "transactionComplete": False,
    "flwRef": "",
    "txRef": "",
    "chargecode": '00',
    "status": "",
    "vbvcode": "",
    "vbvmessage": "",
    "acctmessage": "",
    "currency": "",
    "chargedamount": 00,
    "chargemessage": ""
}

# All payment subclasses are encrypted classes
class Payment(RaveBase):
    """ This is the base class for all the payments """
    def __init__(self, publicKey, secretKey, production, usingEnv):
        # Instantiating the base class
        super(Payment, self).__init__(publicKey, secretKey, production, usingEnv)

    @classmethod
    def retrieve(cls, mapping, *keys): 
        return (mapping[key] for key in keys) 

    @classmethod
    def deleteUnnecessaryKeys(cls,response_dict, *keys):
        for key in keys:
            del response_dict[key]
        return response_dict

    def _preliminaryResponseChecks(self, response, TypeOfErrorToRaise, txRef=None, flwRef=None):
        preliminary_error_response = copy.deepcopy(response_object)
        preliminary_error_response = Payment.deleteUnnecessaryKeys(preliminary_error_response, "transactionComplete", "chargecode", "vbvmessage", "vbvcode", "acctmessage", "currency")

        # Check if we can obtain a json
        try:
            responseJson = response.json()
        except:
            raise ServerError({"error": True, "txRef": txRef, "flwRef": flwRef, "errMsg": response})


        # Check if the response contains data parameter
        if responseJson.get("data", None):
            if txRef:
                flwRef = responseJson["data"].get("flwRef", None)
            if flwRef:
                txRef = responseJson["data"].get("txRef", None)
        else:
            raise TypeOfErrorToRaise({"error": True, "txRef": txRef, "flwRef": flwRef, "errMsg": responseJson.get("message", "Server is down")})
        
        # Check if it is returning a 200
        if not response.ok:
            errMsg = responseJson["data"].get("message", None)
            raise TypeOfErrorToRaise({"error": True, "txRef": txRef, "flwRef": flwRef, "errMsg": errMsg})
        
        return {"json": responseJson, "flwRef": flwRef, "txRef": txRef}

    def _handleChargeResponse(self, response, txRef, request=None, isMpesa=False):
        """ This handles transaction charge responses """

        # If we cannot parse the json, it means there is a server error
        res = self._preliminaryResponseChecks(response, TransactionChargeError, txRef=txRef)
        
        
        responseJson = res["json"]

        if isMpesa:
            return {
                "error": False, 
                "status": responseJson["status"], 
                "validationRequired": True, 
                "txRef": txRef, 
                "flwRef": responseJson["data"]["flwRef"], 
                "narration": responseJson["data"]["narration"]
                }
        else:
            # if all preliminary tests pass
            if not (responseJson["data"].get("chargeResponseCode", None) == "00"):
                if responseJson.get("message", 'None') == 'Momo initiated':
                    return {
                            "error": False, 
                            "status": responseJson["status"],  
                            "message": responseJson["message"],
                            "code": responseJson["data"]["code"],
                            "transaction status": responseJson["data"]["status"],
                            "ts": responseJson["data"]["ts"],
                            "link": responseJson["data"]["link"]
                        }

                return {"error": False, "status": responseJson["status"],"validationRequired": True, "txRef": txRef, "flwRef": responseJson["data"]["flwRef"], "chargeResponseMessage": responseJson["data"]["chargeResponseMessage"]}
            else:
                return {"error": True,  "validationRequired": False, "txRef": txRef, "flwRef": responseJson["data"]["flwRef"]}
    
    def _handleCaptureResponse(self, response, request=None):
        """ This handles transaction charge responses """

        # If we cannot parse the json, it means there is a server error
        res = self._preliminaryResponseChecks(response, PreauthCaptureError, txRef='')
        
        responseJson = res["json"]
        flwRef = responseJson["data"]["flwRef"]
        txRef = responseJson["data"]["txRef"]
        
        # if all preliminary tests pass
        if not (responseJson["data"].get("chargeResponseCode", None) == "00"):
            return {"error": False,  "validationRequired": True, "txRef": txRef, "flwRef": flwRef}
        else:
            return {"error": False, "status":responseJson["status"], "message": responseJson["message"],  "validationRequired": False, "txRef": txRef, "flwRef": flwRef}


    # This can be altered by implementing classes but this is the default behaviour
    # Returns True and the data if successful
    def _handleVerifyResponse(self, response, txRef):
        """ This handles all responses from the verify call.\n
             Parameters include:\n
            response (dict) -- This is the response Http object returned from the verify call
         """
        verify_response = copy.deepcopy(response_object)
        res = self._preliminaryResponseChecks(response, TransactionVerificationError, txRef=txRef)


        responseJson = res["json"]
        # retrieve necessary properties from response 
        verify_response["status"] = responseJson['status']
        verify_response['flwRef'], verify_response["txRef"], verify_response["vbvcode"], verify_response["vbvmessage"], verify_response["acctmessage"], verify_response["currency"], verify_response["chargecode"], verify_response["amount"], verify_response["chargedamount"], verify_response["chargemessage"] = Payment.retrieve(responseJson['data'], "flwref", "txref", "vbvcode", "vbvmessage", "acctmessage", "currency", "chargecode", "amount", "chargedamount", "chargemessage")

        # Check if the chargecode is 00
        if verify_response['chargecode'] == "00":
            verify_response["error"] = False
            verify_response["transactionComplete"] = True
            return verify_response
        
        else:
            verify_response["error"] = True # changed to True on 15/10/2018
            verify_response["transactionComplete"] = False
            return verify_response
        
        # # Check if the chargecode is 00
        # if not (responseJson["data"].get("chargecode", None) == "00"):
        #     return {"error": False, "transactionComplete": False, "txRef": txRef, "flwRef":flwRef}
        
        # else:
        #     return {"error": False, "transactionComplete": True, "txRef": txRef, "flwRef":flwRef}

    
    # returns true if further action is required, false if it isn't    
    def _handleValidateResponse(self, response, flwRef, request=None):
        """ This handles validation responses """

        # If json is not parseable, it means there is a problem in server
            
        res = self._preliminaryResponseChecks(response, TransactionValidationError, flwRef=flwRef)

        responseJson = res["json"]
        if responseJson["data"].get("tx") == None:
            txRef = responseJson["data"]["txRef"]
        else:
            txRef = responseJson["data"]["tx"]["txRef"]

        # Of all preliminary checks passed
        if not (responseJson["data"].get("tx", responseJson["data"]).get("chargeResponseCode", None) == "00"):
            errMsg = responseJson["data"].get("tx", responseJson["data"]).get("chargeResponseMessage", None)
            raise TransactionValidationError({"error": True, "txRef": txRef, "flwRef": flwRef , "errMsg": errMsg})

        else:
            return {"status": responseJson["status"], "message": responseJson["message"], "error": False, "txRef": txRef, "flwRef": flwRef}


    # Charge function (hasFailed is a flag that indicates there is a timeout), shouldReturnRequest indicates whether to send the request back to the _handleResponses function
    def charge(self, feature_name, paymentDetails, requiredParameters, endpoint, shouldReturnRequest=False, isMpesa=False):
        """ This is the base charge call. It is usually overridden by implementing classes.\n
             Parameters include:\n
            paymentDetails (dict) -- These are the parameters passed to the function for processing\n
            requiredParameters (list) -- These are the parameters required for the specific call\n
            hasFailed (boolean) -- This is a flag to determine if the attempt had previously failed due to a timeout\n
            shouldReturnRequest -- This determines whether a request is passed to _handleResponses\n
        """
        # Checking for required components
        try:
            checkIfParametersAreComplete(requiredParameters, paymentDetails)
        except: 
            raise
        
        # Performing shallow copy of payment details to prevent tampering with original
        paymentDetails = copy.copy(paymentDetails)
        
        # Adding PBFPubKey param to paymentDetails
        paymentDetails.update({"PBFPubKey": self._getPublicKey()})

        # Collating request headers
        headers = {
            'content-type': 'application/json',
        }
        if "token" in paymentDetails:
            paymentDetails.update({"SECKEY": self._getSecretKey()})
            # print(json.dumps(paymentDetails))
            response = requests.post(endpoint, headers=headers, data=json.dumps(paymentDetails))
        else:
            # Encrypting payment details (_encrypt is inherited from RaveEncryption)
            encryptedPaymentDetails = self._encrypt(json.dumps(paymentDetails))
            
            # Collating the payload for the request
            payload = {
                "PBFPubKey": paymentDetails["PBFPubKey"],
                "client": encryptedPaymentDetails,
                "alg": "3DES-24"
            }
            response = requests.post(endpoint, headers=headers, data=json.dumps(payload))
            
            #feature logging
            if response.ok:
                tracking_endpoint = self._trackingMap
                responseTime = response.elapsed.total_seconds()
                tracking_payload = {"publicKey": self._getPublicKey(),"language": "Python v2", "version": "1.2.5", "title": feature_name, "message": responseTime}
                tracking_response = requests.post(tracking_endpoint, data=json.dumps(tracking_payload))
            else:
                tracking_endpoint = self._trackingMap
                responseTime = response.elapsed.total_seconds()
                tracking_payload = {"publicKey": self._getPublicKey(),"language": "Python v2", "version": "1.2.5", "title": feature_name + "-error", "message": responseTime}
                tracking_response = requests.post(tracking_endpoint, data=json.dumps(tracking_payload))
        
        if shouldReturnRequest:
            if isMpesa:
                return self._handleChargeResponse(response, paymentDetails["txRef"], paymentDetails, True)
            return self._handleChargeResponse(response, paymentDetails["txRef"], paymentDetails)
        else:
            if isMpesa:
                return self._handleChargeResponse(response, paymentDetails["txRef"], paymentDetails, True)
            return self._handleChargeResponse(response, paymentDetails["txRef"])
        
       

    def validate(self, feature_name, flwRef, otp, endpoint=None):
        """ This is the base validate call.\n
             Parameters include:\n
            flwRef (string) -- This is the flutterwave reference returned from a successful charge call. You can access this from action["flwRef"] returned from the charge call\n
            otp (string) -- This is the otp sent to the user \n
        """

        if not endpoint: 
            endpoint = self._baseUrl + self._endpointMap["account"]["validate"]
            
        # Collating request headers
        headers = {
            'content-type': 'application/json',
        }
        
        payload = {
            "PBFPubKey": self._getPublicKey(),
            "transactionreference": flwRef, 
            "transaction_reference": flwRef,
            "otp": otp
        }
        
        response = requests.post(endpoint, headers = headers, data=json.dumps(payload))

        #feature logging
        if response.ok:
            tracking_endpoint = self._trackingMap
            responseTime = response.elapsed.total_seconds()
            tracking_payload = {"publicKey": self._getPublicKey(),"language": "Python v2", "version": "1.2.5", "title": feature_name, "message": responseTime}
            tracking_response = requests.post(tracking_endpoint, data=json.dumps(tracking_payload))
        else:
            tracking_endpoint = self._trackingMap
            responseTime = response.elapsed.total_seconds()
            tracking_payload = {"publicKey": self._getPublicKey(),"language": "Python v2", "version": "1.2.5", "title": feature_name + "-error", "message": responseTime}
            tracking_response = requests.post(tracking_endpoint, data=json.dumps(tracking_payload))

        return self._handleValidateResponse(response, flwRef)
        
    # Verify charge
    def verify(self, feature_name, txRef, endpoint=None):
        """ This is used to check the status of a transaction.\n
             Parameters include:\n
            txRef (string) -- This is the transaction reference that you passed to your charge call. If you didn't define a reference, you can access the auto-generated one from payload["txRef"] or action["txRef"] from the charge call\n
        """
        if not endpoint:
            endpoint = self._baseUrl + self._endpointMap["verify"]

        # Collating request headers
        headers = {
            'content-type': 'application/json',
        }

        # Payload for the request headers
        payload = {
            "txref": txRef,
            "SECKEY": self._getSecretKey()
        }
        response = requests.post(endpoint, headers=headers, data=json.dumps(payload))

        #feature logging
        if response.ok:
            tracking_endpoint = self._trackingMap
            responseTime = response.elapsed.total_seconds()
            tracking_payload = {"publicKey": self._getPublicKey(),"language": "Python v2", "version": "1.2.5", "title": feature_name, "message": responseTime}
            tracking_response = requests.post(tracking_endpoint, data=json.dumps(tracking_payload))
        else:
            tracking_endpoint = self._trackingMap
            responseTime = response.elapsed.total_seconds()
            tracking_payload = {"publicKey": self._getPublicKey(),"language": "Python v2", "version": "1.2.5", "title": feature_name + "-error", "message": responseTime}
            tracking_response = requests.post(tracking_endpoint, data=json.dumps(tracking_payload))

        return self._handleVerifyResponse(response, txRef)

    #Refund call
    def refund(self, feature_name, flwRef):
        """ This is used to refund a transaction from any of Rave's component objects.\n 
             Parameters include:\n
            flwRef (string) -- This is the flutterwave reference returned from a successful call from any component. You can access this from action["flwRef"] returned from the charge call
        """
        payload = {
            "ref": flwRef,
            "seckey": self._getSecretKey(),
        }
        headers = {
            "Content-Type":"application/json"
        }
        endpoint = self._baseUrl + self._endpointMap["refund"]

        response = requests.post(endpoint, headers = headers, data=json.dumps(payload))

        #feature logging
        if response.ok:
            tracking_endpoint = self._trackingMap
            responseTime = response.elapsed.total_seconds()
            tracking_payload = {"publicKey": self._getPublicKey(),"language": "Python v2", "version": "1.2.5", "title": feature_name, "message": responseTime}
            tracking_response = requests.post(tracking_endpoint, data=json.dumps(tracking_payload))
        else:
            tracking_endpoint = self._trackingMap
            responseTime = response.elapsed.total_seconds()
            tracking_payload = {"publicKey": self._getPublicKey(),"language": "Python v2", "version": "1.2.5", "title": feature_name + "-error", "message": responseTime}
            tracking_response = requests.post(tracking_endpoint, data=json.dumps(tracking_payload))

        try:
            responseJson = response.json()
        except ValueError:
            raise ServerError(response)
        
        if responseJson.get("status", None) == "error":
            raise RefundError(responseJson.get("message", None))
        elif responseJson.get("status", None) == "success":
            return True, responseJson.get("data", None)


        





import requests, json, copy
from rave_python.rave_base import RaveBase
from rave_python.rave_misc import checkIfParametersAreComplete, generateTransactionReference
from rave_python.rave_exceptions import  ServerError, IncompletePaymentDetailsError, PlanCreationError, PlanStatusError

class Subscriptions(RaveBase) :
    def __init__(self, publicKey, secretKey, production, usingEnv):
        self.headers = {
            'content-type': 'application/json'
        }
        super(Subscriptions, self).__init__(publicKey, secretKey, production, usingEnv)
    
    def _preliminaryResponseChecks(self, response, TypeOfErrorToRaise, name):
        # Check if we can obtain a json
        try:
            responseJson = response.json()
        except:
            raise ServerError({"error": True, "name": name, "errMsg": response})

        # Check if the response contains data parameter
        if not responseJson.get("data", None):
            raise TypeOfErrorToRaise({"error": True, "name": name, "errMsg": responseJson.get("message", "Server is down")})
        
        # Check if it is returning a 200
        if not response.ok:
            errMsg = responseJson["data"].get("message", None)
            raise TypeOfErrorToRaise({"error": True, "errMsg": errMsg})
        
        return responseJson
        
    
    # This makes and handles all requests pertaining to the status of your payment plans
    def _handlePlanStatusRequests(self, type, endpoint, isPostRequest=False, data=None):

        # Checks if it is a post request
        if isPostRequest:
            response = requests.post(endpoint, headers=self.headers, data=json.dumps(data))
        else:
            response = requests.get(endpoint, headers=self.headers)
    
        # Checks if it can be parsed to json
        try:
            responseJson = response.json()
        except:
            raise ServerError({"error": True, "errMsg": response.text })

        # Checks if it returns a 2xx code
        if response.ok:
            return {"error": False, "returnedData": responseJson}
        else:
            raise PlanStatusError(type, {"error": True, "returnedData": str(responseJson) })

    #gets all subscriptions connected to a merchant's account
    def all(self):
        endpoint = self._baseUrl + self._endpointMap["subscriptions"]["list"] + "?seckey="+self._getSecretKey()
        return self._handlePlanStatusRequests("List", endpoint)
    
    def fetch(self, subscription_id=None, subscription_email=None):
        if subscription_id:
            endpoint = self._baseUrl + self._endpointMap["subscriptions"]["fetch"] + "?seckey="+self._getSecretKey() + "&id="+str(subscription_id)
        elif subscription_email:
            endpoint = self._baseUrl + self._endpointMap["subscriptions"]["fetch"] + "?seckey="+self._getSecretKey() + "&1="+subscription_email
        else:
            return "You must pass either plan id or plan name in order to fetch a plan's details"
        return self._handlePlanStatusRequests("Fetch", endpoint)
    
    def cancel(self, subscription_id):
        if not subscription_id:
            return "Subscription id was not supplied. Kindly supply one"
        endpoint = self._baseUrl + self._endpointMap["subscriptions"]["cancel"] + str(subscription_id) + "/cancel"
        data = {"seckey": self._getSecretKey()}
        return self._handlePlanStatusRequests("Cancel", endpoint, isPostRequest=True, data=data)
    
    # activates a subscription plan
    # Params
    # id: subscription_id *required
    def activate(self, subscription_id):
        if not subscription_id:
            return "Subscription id was not supplied. Kindly supply one"
        endpoint = self._baseUrl + self._endpointMap["subscriptions"]["activate"] + str(subscription_id) + "/activate"
        data = {"seckey": self._getSecretKey()}
        return self._handlePlanStatusRequests("Activat", endpoint, isPostRequest=True, data=data)
